/****************************************************************************************
                               DEMAND INTELLIGENCE

NAME:          di_etl_export_sales_forecast

DESCRIPTION:   This macro is used to export sales forecast.
               Markdown and Promotion solutions
               are involved in this export.

APPLICATIONS:  All

MODULE:        DM

INPUTS:        fcst_prod_id= Product ID (get forecast for all products below this hierarchy)
               fcst_geo_id=  Geography ID (get forecast for all stores below this hierarchy)
               fcst_model_grps= Model Group (get forecasts for these model groups
               fcst_weeks=  Get forecast for number of weeks from today

OUTPUTS:      _forecast_export_table - table created with forecast results for given input parameters
GLOBAL VAR:    NA
USAGE:
THREADED:      NA
DOCUMENT:      NA

                DAILY_MODEL  = 1 (Use Daily Model)  0 (Do not use Daily Model)
                DAILY_OUTPUT = 1 (Output daily forecast)  0 (Output weekly forecast)


COMMENTS:
AUTHOR:        vekada  Kadari, Venu
SUPPORT:       vekada Kadari, Venu

HISTORY:       04Jun2010 vekada   Rewritten for TKMI Solution
               25Jan2011 vekada   REG_PRICE field added
               03Aug2011 vekada   Enhancements for 5.2
	       13Sep2016 sinsro   DEFECT S1270364: CRP:di_etl_export_sales_forecast does not 
	                          raise errors from MerchIntelGridSvr

****************************************************************************************/

%macro di_etl_export_sales_forecast
(
   fcst_prod_id=,
   fcst_geo_id=,
   fcst_weeks=,
   _geo_hier_assoc_cd=,
    m_etlp_units_fmt=,
    m_etlp_price_fmt=,
    m_etlp_pct_fmt=,

   model_group_ids_table=,
   geo_prod_ids_table=,
   geo_prod_sks_table=,
   daily_model=,
   daily_output=,

   _user=,
   _pwd=,
   _getPWFromDomain=YES,

   _forecast_export_table=,
   code_line_length=%str($60),
   get_currency_id=,
   m_debug=1
);

%global sql_statement;

/***************************************************************************/
/**     Create structure with formats for the table to be exported        **/
/**     Create at the beginning to prevent exporting previous results     **/
/***************************************************************************/


%di_util_assert_sym(sym=_forecast_export_table,
                    msg=The parameter _forecast_export_table cannot be blank,
                    msgtype=ERROR);

/***************************************************************************/
/**                Read plans from stg_plan_export table                  **/
/***************************************************************************/
/* Check if the table exist */
%let plan_count=0;
%if %sysfunc(exist(didm_stg.stg_plan_export)) %then
%do;
   proc sql noprint;
      select count(*)
         into :plan_count
         from didm_stg.stg_plan_export;
   quit;

%end;


/* Create structure for geo prod */
data  work._sf_geo_prod_filter;
   attrib geo_hier_sk       length=4;
   attrib prod_hier_sk      length=6;
stop; run;

/*****************************************************************************/
/**      Get lowest prod, geo and date levels from di_data.data_lvl.        **/
/**      The GEO_PROD_LOW_DATA_LVL setting in di_data.global_settings       **/
/**      table determins the data_lvl_id                                    **/
/*****************************************************************************/
   proc sql noprint;

      %local  low_date_lvl;

      select   date_lvl

         into  :low_date_lvl

         from  di_data.global_setting gs,
               di_data.data_lvl       dl

         where input(gs.spec_value,8.) = dl.data_lvl_id
         and   upcase(gs.spec_nm)  = 'GEO_PROD_LOW_DATA_LVL'
         and   upcase(gs.app)      = 'ALL'
      ;
   quit;

   %di_util_assert_sym(sym=low_date_lvl,msg=low_date_lvl could not be determined.  Check di_data.global_setting and di_data.data_lvl.);

   %put NOTE- low_date_lvl=&low_date_lvl;

   /* Let default promotion geography hierarchy be 1 */
   %let promo_geo_hier=1;
   proc sql noprint;

      select   gs.spec_value
          into  :promo_geo_hier
          from  di_data.global_setting gs

         where upcase(gs.spec_nm)  = 'PROMO_GEO_HIER'
         and   upcase(gs.app)      = 'APP'
      ;
   quit;
   %put NOTE: Geography hierarchy for promotions promo_geo_hier=&promo_geo_hier;

   /* Get prod_partition_lvl */
   proc sql noprint;
      select spec_value into :m_prod_partition_lvl
         from di_data.global_setting
         where SPEC_NM="PROD_PARTITION_LVL"

      ;
   quit;

   %di_util_assert_sym(sym=m_prod_partition_lvl,msg=PROD_PARTITION_LVL missing.  Check di_data.global_setting.);
   %let m_prod_partition_lvl=&m_prod_partition_lvl;

   * Prod partition level is blank;
   %if &m_prod_partition_lvl= %then %do;
      %put ERROR: PROD_PARTITION_LVL is empty. Check di_data.global_setting.;
      %return;
   %end;


/*****************************************************************************/
/** For given plans in didm_stg.stg_plan_export table find geo/prods.       **/
/** If didm_stg.stg_plan_export has no records then get prod_hier_sk and    **/
/** geo_hier_sk for given fcst_prod_id and fcst_geo_id in the               **/
/** parameters table.  Expand to the lowest level to get all the prod/geos. **/
/*****************************************************************************/

%if (&plan_count>0) %then
%do;

   /* Display NOTE if prod id and geo id are also present */
   %if (%length(&fcst_prod_id)>0 || %length(&fcst_geo_id)>0 ) %then
   %do;
      %put NOTE: Plans specified in didm_stg.stg_plan_export table will be used for export.;
      %put NOTE- The parameters fcst_prod_id=&fcst_prod_id and fcst_geo_id=&fcst_geo_id will be ignored.;
   %end;

   /** Push didm_stg.stg_plan_export to di_trans.stg_plan_export **/

   /*---- Drop table di_trans.stg_plan_export ----*/
   %put %str(NOTE: Dropping table di_trans.stg_plan_export);
   proc datasets lib = di_trans nolist nowarn memtype = (data view);
      delete stg_plan_export;
   quit;

   /*---- Create a new table  ----*/
   data di_trans.stg_plan_export;
      attrib plan length = $100;
      attrib app  length = $5;
      call missing(of _all_);
      stop;
   run;

   proc append base = di_trans.stg_plan_export
      data = didm_stg.stg_plan_export force ;
      where app="CPO";
   run;

     /*
                     Get Geo-Prods for Markdown plans

      This condition plan.plan_status_cd=10 was removed from query below to allow inactive plans.  This is a way
      to specify any geo-prod combinations for forecast extract. There is really
      no value in imposing this additional condition of plan_status_cd=10 and
      there is no benefit in going to mdo_plan_group table.

      */

      /* Get geo prods related to markdown application */

     %let sql_statement = %str(
      select
                    mpm.geo_hier_sk,
                    mpm.prod_hier_sk

      from          di_trans.mdo_plan plan

      inner join    di_trans.stg_plan_export spe
      on            spe.plan=plan.plan_nm

      inner join    di_trans.mdo_plan_member mpm
      on            mpm.mdo_plan_group_sk=plan.mdo_plan_group_sk

    );

     %let sql_statement = %sysfunc(COMPBL(&sql_statement));
     %put &sql_statement;

      %di_dm_trans_exe(pass_through_sql=%quote(&sql_statement),
                _output_table = cpo_geo_prods,
                debug_flag=1,
                session_flg=1);


      /* Get geo prods related to Promotion application */

   /*---- Drop table di_trans.stg_plan_export ----*/
   %put %str(NOTE: Dropping table di_trans.stg_plan_export);
   proc datasets lib = di_trans nolist nowarn memtype = (data view);
      delete stg_plan_export;
   quit;

   /*---- Create a new table  ----*/
   data di_trans.stg_plan_export;
      attrib plan length = $100;
      attrib app  length = $5;
      call missing(of _all_);
      stop;
   run;

   proc append base = di_trans.stg_plan_export
      data = didm_stg.stg_plan_export force ;
      where app="RPP";
   run;

     /*
                     Get Geo-Prods for Promotion plans

      The condition rpp_plan.status_cd=2 is removed to
      include all plans (active and inactive).  This is a way to specify any
      geo-prod combinations for forecast extract. There is really no value in imposing
      this additional condition of status_cd=2

     */

      %let sql_statement = %str(
      select        sd.geo_hier_sk,
                    ppa.prod_hier_sk

      from          di_trans.rpp_plan plan

      inner join    di_trans.stg_plan_export spe
      on            spe.plan=plan.plan_id

      inner join    di_trans.rpp_plan_prod_asgmnt ppa
      on            plan.plan_sk=ppa.plan_sk

      inner join    di_trans.scope_detail sd
      on            plan.scope_sk=sd.scope_sk
      );

     %let sql_statement = %sysfunc(COMPBL(&sql_statement));
     %put &sql_statement;

      %di_dm_trans_exe(pass_through_sql=%quote(&sql_statement),
                _output_table = rpp_geo_prods,
                debug_flag=1,
                session_flg=1);



   /*
      Expand to PROD_PARTITION_LVL and no more

   */
   proc sql noprint;
      insert into     work._sf_geo_prod_filter
      select distinct gh.geo_hier_sk,
                      ph.prod_hier_sk

      from            di_data.product_hier_assoc_dm    ph,
                      di_data.geography_hier_assoc_dm  gh

      inner join      cpo_geo_prods plan_gp
      on              plan_gp.geo_hier_sk  = gh.parent_geo_hier_sk
      and             gh.geo_lvl           = gh.parent_geo_lvl

      where           plan_gp.prod_hier_sk = ph.parent_prod_hier_sk
      and             (ph.prod_lvl = &m_prod_partition_lvl or ph.prod_lvl =ph.parent_prod_lvl)
      and             ph.prod_lvl >= &m_prod_partition_lvl
      and             gh.geo_hier_assoc_cd   = 1
      and             ph.prod_hier_assoc_cd  = 1

      ;

   /* Get geo-prod for promotion geo-prods */
      insert into     work._sf_geo_prod_filter
      select distinct gh.geo_hier_sk,
                      ph.prod_hier_sk

      from            di_data.product_hier_assoc_dm    ph,
                      di_data.geography_hier_assoc_dm  gh

      inner join      rpp_geo_prods plan_gp
      on              plan_gp.geo_hier_sk  = gh.parent_geo_hier_sk
      and             gh.geo_lvl           = gh.parent_geo_lvl

      where
                      plan_gp.prod_hier_sk = ph.parent_prod_hier_sk
      and             gh.geo_hier_assoc_cd   = &promo_geo_hier
      and             ph.prod_hier_assoc_cd  = 1
      and             (ph.prod_lvl = &m_prod_partition_lvl or ph.prod_lvl =ph.parent_prod_lvl)
      and             ph.prod_lvl >= &m_prod_partition_lvl

      ;

   quit;



%end;
%else
%do;


   proc sql noprint;

      /* Get prod_hier_sk that matches given fcst_prod_id */
      select   max(prod_hier_sk)
         into  :fcst_prod_hier_sk
         from  di_data.product_dm
         where prod_id = "&fcst_prod_id." ;

      /* Get geo_hier_sk that matches given fcst_geo_id */
      select   max(geo_hier_sk)
         into  :fcst_geo_hier_sk
         from  di_data.geography_dm
         where geo_id = "&fcst_geo_id.";
   quit;

   %put NOTE: No plans specified in didm_stg.stg_plan_export table.;
   %put NOTE- Using parameters fcst_prod_id=&fcst_prod_id and fcst_geo_id=&fcst_geo_id for forecasting.;
   %put NOTE- Using parameters prod_hier_sk=&fcst_prod_hier_sk and geo_hier_sk=&fcst_geo_hier_sk for forecasting.;
   %put NOTE- PROD_PARTITION_LVL=&m_prod_partition_lvl.;
   %put NOTE- GEO_HIER_ASSOC_CD=&_geo_hier_assoc_cd.;



   %if not %symexist(fcst_prod_hier_sk) %then
    %let fcst_prod_hier_sk=;

   %if not %symexist(fcst_geo_hier_sk) %then
    %let fcst_geo_hier_sk=;

   %di_util_assert_sym(sym=fcst_prod_hier_sk,
                       msg=For fcst_prod_id=&fcst_prod_id input in di_mon.job_param table there was no prod_hier_sk in di_data.product_dm table.,
                       msgtype=ERROR);

   %di_util_assert_sym(sym=fcst_geo_hier_sk,
                       msg=For fcst_geo_id=&fcst_geo_id input in di_mon.job_param table there was no geo_hier_sk in di_data.geography_dm table.,
                       msgtype=ERROR);


   /* Get geo-prod filter table
       No longer checking to see if geo-prod combination exists in di_data.geo_prod table
   */
   proc sql noprint;
      insert into     work._sf_geo_prod_filter
      select distinct gh.geo_hier_sk,
                      ph.prod_hier_sk

      from            di_data.product_hier_assoc_dm    ph,
                      di_data.geography_hier_assoc_dm  gh

       where           ph.parent_prod_hier_sk = &fcst_prod_hier_sk and
                      (ph.prod_lvl = &m_prod_partition_lvl or ph.prod_lvl =ph.parent_prod_lvl) and
                      ph.prod_lvl >= &m_prod_partition_lvl        and
                      gh.parent_geo_hier_sk  = &fcst_geo_hier_sk  and
                      gh.geo_lvl             = gh.parent_geo_lvl  and
                      gh.geo_hier_assoc_cd   = &_geo_hier_assoc_cd and
                      ph.prod_hier_assoc_cd  = 1

      ;



   quit;



%end;



   /* Add Geo prods from model group ids table input by user */
   %if (%length(&model_group_ids_table)>2 )  %then
   %do;

        %put NOTE: Model groups included in table &model_group_ids_table will be used as input ;


        /* Geo_prod_hier_sk and prod_hier_sk in model_geo_prod is always at the lowest level */
      %let m_subdir=MODEL_GEO_PROD;
      %let m_partlib=ZMGP_LIB;
      %di_etl_set_lib (parent_lib=di_data, sub_folder=&m_subdir.,partition_lib=&m_partlib. );

      data _null_;
            set &model_group_ids_table end=eof;
            if _n_ eq 1 then do;
               call execute("data _gp_filter_tmp_321 (keep=geo_hier_sk prod_hier_sk); set ");
            end;
            call execute(" &m_partlib..model_geo_prod_" || left(model_group_id) || "");
            if eof then call execute("; run;");
         run;

      proc sql;

           insert into work._sf_geo_prod_filter

           select unique
              GEO_HIER_SK,
              PROD_HIER_SK

           from work._gp_filter_tmp_321

             ;

      quit;

      libname &m_partlib clear;


   %end;

   /* Add Geo prods from tables input by user */
   %else %if ((%length(&geo_prod_ids_table)>2 ) or (%length(&geo_prod_sks_table)>2 ))  %then
   %do;

       /* Create structure for geo prod */
       data work._sf_geo_prod_filter2;
          attrib geo_hier_sk       length=4;
          attrib prod_hier_sk      length=6;
       stop; run;



       %if (%length(&geo_prod_ids_table)>2 ) %then
       %do;
          %put NOTE: Geo prods included in table &geo_prod_ids_table will be added to the input ;


           /* Get SKs from IDs input by user */
           proc sql;

                insert into work._sf_geo_prod_filter2
                select unique
                   GEOGRAPHY_DM.GEO_HIER_SK,
                   PRODUCT_DM.PROD_HIER_SK

                from &geo_prod_ids_table gp_ids

                inner join
                       DI_DATA.GEOGRAPHY_DM GEOGRAPHY_DM
                       on GEOGRAPHY_DM.GEO_ID=gp_ids.GEO_ID

                inner join DI_DATA.geography_hier_assoc_dm ghad
                     on GEOGRAPHY_DM.GEO_HIER_SK= ghad.PARENT_GEO_HIER_SK
                    and ghad.geo_hier_assoc_cd=&_geo_hier_assoc_cd

                inner join DI_DATA.PRODUCT_DM PRODUCT_DM
                  on  PRODUCT_DM.PROD_ID=gp_ids.PROD_ID

                inner join DI_TRANS.product_hier_assoc_dm phad
                   on PRODUCT_DM.PROD_HIER_SK= phad.PARENT_PROD_HIER_SK

                ;

            quit;


       %end;

       %if (%length(&geo_prod_sks_table)>2 ) %then
       %do;
          %put NOTE: Geo prods included in table &geo_prod_sks_table will be added to the input ;

           /* Simple append */
           proc sql;

                insert into work._sf_geo_prod_filter2
                select unique
                   GEO_HIER_SK,
                   PROD_HIER_SK

                from &geo_prod_sks_table

                ;

            quit;



       %end;


           /*
              Expand to PROD_PARTITION_LVL and no more
           */

           proc sql noprint;
              insert into     work._sf_geo_prod_filter
              select distinct gh.geo_hier_sk,
                              ph.prod_hier_sk

              from            di_data.product_hier_assoc_dm    ph,
                              di_data.geography_hier_assoc_dm  gh

              inner join      work._sf_geo_prod_filter2 user_gp
              on              user_gp.geo_hier_sk  = gh.parent_geo_hier_sk
              and             gh.geo_lvl           = gh.parent_geo_lvl

              where           user_gp.prod_hier_sk = ph.parent_prod_hier_sk
              and             (ph.prod_lvl = &m_prod_partition_lvl or ph.prod_lvl =ph.parent_prod_lvl)
              and             ph.prod_lvl >= &m_prod_partition_lvl
              and             gh.geo_hier_assoc_cd   = 1
              and             ph.prod_hier_assoc_cd  = 1

              ;


           /* Get geo-prod for promotion geo-prods */
              insert into     work._sf_geo_prod_filter
              select distinct gh.geo_hier_sk,
                              ph.prod_hier_sk

              from            di_data.product_hier_assoc_dm    ph,
                              di_data.geography_hier_assoc_dm  gh

              inner join      work._sf_geo_prod_filter2 user_gp
              on              user_gp.geo_hier_sk  = gh.parent_geo_hier_sk
              and             gh.geo_lvl           = gh.parent_geo_lvl

              where
                              user_gp.prod_hier_sk = ph.parent_prod_hier_sk
              and             gh.geo_hier_assoc_cd   = &promo_geo_hier
              and             ph.prod_hier_assoc_cd  = 1
              and             (ph.prod_lvl = &m_prod_partition_lvl or ph.prod_lvl =ph.parent_prod_lvl)
              and             ph.prod_lvl >= &m_prod_partition_lvl

              ;

           quit;


   %end;




   /* Remove duplicates.  Duplicates within plans are eliminated in above queries but there could be duplicates because of CPO and RPP plans */
   proc sort data=work._sf_geo_prod_filter nodup;
      by _all_;
   run;

   /* Count to see if there are any geo-prods */
    proc sql noprint;
      select count(*)
      into   :gp_count
      from   _sf_geo_prod_filter;
   quit;





   /* If there are no geo-prod's abort */
   %if &gp_count<=0 %then
   %do;
      %put ERROR: No input geo-prods found;
      %return;
   %end;
   %else
   %do;
      %put NOTE: Number of distinct geo-prod combinations found for given plans in didm_stg.stg_plan_export table is = &gp_count ;
   %end;


   /*---- Drop table di_trans.stg_plan_export ----*/
   %put %str(NOTE: Dropping table di_trans.stg_plan_export);
   proc datasets lib = di_trans nolist nowarn memtype = (data view);
      delete stg_plan_export;
   quit;







   /* Move input data to Trans table */
   %if %sysfunc(exist(DI_TRANS.FCST_EXPORT_INPUT))
   %then %do;
      proc sql noprint ;
          delete * from di_trans.fcst_export_input;
          insert into di_trans.fcst_export_input
             (GEO_HIER_SK, PROD_HIER_SK, GEO_HIER_ASSOC_CD,
             UNCONSTR_FLG, FCST_WEEKS, DAILY_MODEL, DAILY_OUTPUT)

          select unique
             geo_hier_sk,
             prod_hier_sk,
             &_geo_hier_assoc_cd as geo_hier_assoc_cd,
             1,
             &fcst_weeks as fcst_weeks,
             &daily_model,
             &daily_output

          from work._sf_geo_prod_filter gpfilter

         ;
      quit;
   %end;
   %else %do;
      %put ERROR: Table  DI_TRANS.FCST_EXPORT_INPUT not found.  Aborting!;
      %return;

   %end;









/* Figure out date_sk's for forecast period */

/* Find out current date_sk from global_setting table */
proc sql noprint;
   select   input(spec_value,8.)
     into   :curr_date_sk
   from     di_data.global_setting
   where    upcase(spec_nm) = 'CURRENT_DATE_SK'
      and   upcase(app) = 'ALL';

   /* Find fiscal_start_dt for current date_sk */
   select  fiscal_start_dt*1
   into    :curr_dt
   from    di_data.date_dm
   where   date_sk = &curr_date_sk.;

   %let fcst_end_dt=%eval(&curr_dt+7*&fcst_weeks-1);

   /* Find out date_sk corresponding to end date of forecast period */
   /* This date_sk will be 0 if forecast period exceeds the dates stored */
   %let end_date_sk=0;
   select  date_sk
   into    :end_date_sk
   from    di_data.date_dm
   where   fiscal_end_dt=&fcst_end_dt
   and     date_lvl=&low_date_lvl;


   /* If end_date_sk is not found from above then use the max date_sk for given low_date_lvl*/
   %if (end_date_sk=0) %then %do;
      select max(date_sk)
      into   :end_date_sk
      from   di_data.date_dm
      where  date_lvl=&low_date_lvl;

      %put WARN: Forecast period end date of %sysfunc(putn(&fcst_end_dt, date9)) is not in date dimension table;
      %put WARN- Using maximum date_sk of &end_date_sk stored in the date dimension table;

   %end;

   %put NOTE:Forecast period is %sysfunc(putn(&curr_dt, date9)) to %sysfunc(putn(&fcst_end_dt, date9));
   %put NOTE:Forecast period: begin curr_date_sk=&curr_date_sk, end end_date_sk=&end_date_sk;
quit;



/*
   DI_DATA is a standard SAS library in all retail solutions.  This library is
   assumed to be assigned. A folder called "fcst_export" within DI_DATA library
   has results from running Forecast Export.  This macro removes previous results.
   If the folder doesn't exist it creates one.

*/

   %local from_lib_path;

   /* Get path to the library */
   proc sql noprint;
      select path into :from_lib_path
      from sashelp.vlibnam
      where upcase(libname) = "DI_DATA"
      and level in (0,1);
   quit;

   %let fcst_export_path = %di_util_create_folder(relative_dir=fcst_export,parent_dir=&from_lib_path);
   libname _fcstexp  "&fcst_export_path" %di_dio_libname;

   /*
      Delete all members in this library before making a request to get forecasts.
      These forecasts being deleted are from previous run of forecast extract
   */


   proc datasets library=_fcstexp kill;
   quit; run;


    /* Get all the Products at product partition level */
    proc sql noprint;
        create table work._prod_partitions as
           select distinct ph.parent_prod_hier_sk as prod_hier_sk

           from            di_data.product_hier_assoc_dm    ph

           inner join      work._sf_geo_prod_filter fcst_input
           on              fcst_input.prod_hier_sk  = ph.prod_hier_sk

           where           ph.parent_prod_lvl = &m_prod_partition_lvl
           and             ph.prod_hier_assoc_cd  = 1

           ;

    quit;




   /* Make a request to the server to compute forecasts */
   %di_tkmi_request(_cmd=%str(fcst_export;),
                    captainDir=fcst_export,
                    prod_hier_sk_table=work._prod_partitions,
                    serverDir=fcst_export,
                    _rc=,
                    _user=&_user,
                    _pwd=&_pwd,
                    _getPWFromDomain=&_getPWFromDomain,
                     debug=&m_debug) ;

%put processRequestReturn=&processRequestReturn;

/*-------------------------------------------------------------------------------------------------------------------------*/
/* DEFECT S1270364: CRP:di_etl_export_sales_forecast does not raise errors from MerchIntelGridSvr. Modified the data step  */
/* below to capture the status code from response file. The status is sent within the <status> tag. status=0,1 indicates   */
/* successful and warning respectively and 2 is for error. The macro variable processRequestReturn will have the response  */
/* status and is generated in di_tkmi_request                                                                              */
/* if status=2, no further processing happens                                                                              */
/*-------------------------------------------------------------------------------------------------------------------------*/
   
%if &processRequestReturn = 2 %then %do;
   %put "ERROR:  An error occurred in forecast export request, please review the MerchIntelGridSvr log ";
   %put "ERROR:  Aborting forecast export!";
   %let syscc = 9999;
   %return;
%end;
   %local fcst_count;
   %let fcst_count=0;

   /* Count how many forecast extract tables there are */
   proc sql noprint;
      select count(*)
         into :fcst_count
         from sashelp.vtable
         where Libname eq "_FCSTEXP";
   quit;

   /* Error and get out if there are no forecast tables */
   data _null_;
      if &fcst_count=0 then do;
         put "ERROR:  No forecast results available in _FCSTEXP library - &fcst_export_path";
         put "ERROR-  Aborting forecast export!";
      end;
   run;



   /*
     Appends all the forecasts in folder fcst_export into a work table.
          Creates something like this
          data work._forecasts   ;
          set
            _FCSTEXP.FCST_EXPORT_9_1
            _FCSTEXP.FCST_EXPORT_9_2
            _FCSTEXP.FCST_EXPORT_9_3  ;
          run;


   */
   %if &fcst_count>0 %then %do;
      %let CRLF='0D0A'x ;

      proc sql;
         create table work._fcst_files as
         select libname, memname from
         sashelp.vtable
         where libname eq "_FCSTEXP"
         ;
      quit;


      /* If filename is blank then give it some name */
       %if (%symexist(_codeGenFile) ) %then
      %do;
         %let _codeGenFile=&_codeGenFile;
      %end;
      %else %do;
         %let _codeGenFile=%di_util_first_libpath(work);
         %let _codeGenFile=%nrbquote(&_codeGenFile./append_fcst_extract_01.sas);
      %end;


      /* Number of datasets to use in a set statement at a time */
       %if (%symexist(_numSetStatement) ) %then
      %do;
         %let _numSetStatement=&_numSetStatement;
      %end;
      %else %do;
         %let _numSetStatement=50;
      %end;


      /* _append_method is SAS_FILE or SAS_MACRO */
       %if (%symexist(_append_method) ) %then
      %do;
         %let _append_method=&_append_method;
      %end;
      %else %do;
         %let _append_method=SAS_FILE;
      %end;

      /* Use a default length of 200 characters */
       %if (%symexist(_Line_Length) ) %then
      %do;
         %let _Line_Length=&_Line_Length;
      %end;
      %else %do;
         %let _Line_Length=200;
      %end;

     %put NOTE: _codeGenFile=&_codeGenFile;
     %put NOTE: _append_method=&_append_method;


      FILENAME EXTRACT    "&_codeGenFile";

       %if "&_append_method"="SAS_FILE" %then
       %do;

          /*
              Write out generated code to an external file
          */

          data _null_;
             set _fcst_files end=last;
             attrib code_line length=&code_line_length;
            FILE EXTRACT LRECL=&_Line_Length.;

             if _n_ = 1 then do;
                code_line="data work._forecasts ;" || &CRLF || "     set ";
               put code_line ;
             end;

             code_line=("     " || trim(libname) || "." || trim(memname) || "  ")  ;
             put code_line;

             if (last) then do;
                code_line=("; run;"  || &CRLF )  ;
               put code_line ;
             end;
           else if (mod(_n_,&_numSetStatement)=0) then do;
                code_line=("; run;"  || &CRLF )  ;
               put code_line ;
                code_line="data work._forecasts ;" || &CRLF || "     set work._forecasts";
               put code_line ;
            end;

         run;


          /* Run sas code saved in this file.  This extract pricing rules to SAS datasets */
          %include "&_codeGenFile.";

       %end;
       %else
       %do;

          data _null_;
             set _fcst_files end=last;

             if _n_ = 1 then do;
                call symput("procappend", " data work._forecasts ;" || &CRLF || "     set ");
             end;

             call symput("procappend", symget("procappend") || trim(libname) || "." || trim(memname) || "  ")  ;

             if (last) then do;
                call symput("procappend", symget("procappend") || "; run;"  || &CRLF )  ;
             end;

         run;

         * execute code to append all the forecast results ;
         &procappend;


      %end;




   %end;
   %else %do;
      %return;
   %end;


   /*
      Get maximum length of prod_id and geo_id.  Field size reduction helps greatly when there are large number of rows.
   */

   %let max_len_prod_id=;
   %let max_len_geo_id=;

   proc sql;
      select max(length(prod_id))
         into :max_len_prod_id
         from di_data.product_dm
        ;


      select max(length(geo_id))
         into :max_len_geo_id
         from di_data.geography_dm
        ;

   quit;

   %put NOTE: Maximum length of prod_id field in product_dm table is &max_len_prod_id.;
   %put NOTE- This length of &max_len_prod_id will be used for prod_id in output dataset to conserve space.;
   %put NOTE- Please ignore WARNING: Multiple lengths were specified for the variable prod_id by ;
   %put NOTE- input data set(s).  This may cause truncation of data.;
   %put  ;
   %put NOTE: Maximum length of geo_id field in geography_dm table is &max_len_geo_id.;
   %put NOTE- This length of &max_len_geo_id will be used for geo_id in output dataset to conserve space.;
   %put NOTE- Please ignore WARNING: Multiple lengths were specified for the variable geo_id by ;
   %put NOTE- input data set(s).  This may cause truncation of data. ;


   /*
      Get all the results into a format required export
      Lookup prod_id and geo_id
   */

   data &_forecast_export_table;
         attrib start_dt                      length=8   informat=date9. format=date9.;
         attrib end_dt                        length=8   informat=date9. format=date9.;
         attrib forecast_dt                   length=8   informat=date9. format=date9.;
         attrib prod_id                       length=$   &max_len_prod_id;
         attrib geo_id                        length=$   &max_len_geo_id;
         attrib base_forecast_units           length=8   &m_etlp_units_fmt ;
         attrib sales_forecast_units          length=8   &m_etlp_units_fmt ;
         attrib unconstr_sales_forecast_units length=8   &m_etlp_units_fmt ;
         attrib beg_inv_units                 length=8   &m_etlp_units_fmt ;
         attrib effective_price               length=8   &m_etlp_price_fmt;
         attrib effective_discount_pct        length=8   &m_etlp_pct_fmt;
         attrib perm_price                    length=8   &m_etlp_price_fmt;
         attrib reg_price                     length=8   &m_etlp_price_fmt;
         attrib reg_cost                      length=8   &m_etlp_price_fmt;
         attrib con_cost_amt                  length=8   &m_etlp_price_fmt;
         attrib uncon_cost_amt                length=8   &m_etlp_price_fmt;

      set work._forecasts(rename=(uncon_str_sales_forecast_units=unconstr_sales_forecast_units
                            eff_price=effective_price eff_discount_pct=effective_discount_pct)) ;

      keep start_dt end_dt forecast_dt prod_id geo_id base_forecast_units sales_forecast_units unconstr_sales_forecast_units
                     beg_inv_units effective_price effective_discount_pct perm_price reg_price reg_cost con_cost_amt uncon_cost_amt;



      /* Get prod_id */
      set di_data.product_dm key=prod_hier_sk;
         if _iorc_ ne 0 then
         do;
           /*
               This condition should never occur.
               If it did occur, it is better to move on rather than stop.
            */
         end;

         _iorc_=0;  _error_=0;

      /* get geo_id */
      set di_data.geography_dm key=geo_hier_sk;
         if _iorc_ ne 0 then
         do;
           /*
               This condition should never occur.
               If it did occur, it is better to move on rather than stop.
            */
         end;

         _iorc_=0;  _error_=0;


      forecast_dt=&fcst_end_dt;
   run;


   /* Drop work._forecasts table */
   proc datasets nolist lib=work;
      delete _forecasts;
   quit;


   %if  ("&get_currency_id"="1") %then
   %do;
       proc sql noprint;
          create table work.currency_dm_work as
          select curr.geo_hier_sk, curr.currency_id, gdm.geo_id
          from di_dm2.currency_dm curr
          inner join di_dm2.geography_dm gdm
          on curr.geo_hier_sk=gdm.geo_hier_sk
          ;

       quit;

       proc sort data=work.currency_dm_work;
          by geo_id;
       run;

       proc sql;
          alter table work.currency_dm_work add CONSTRAINT PRIM_KEY PRIMARY KEY (GEO_ID);
       quit;

      data &_forecast_export_table;

         set &_forecast_export_table ;

         keep start_dt end_dt forecast_dt prod_id geo_id base_forecast_units sales_forecast_units unconstr_sales_forecast_units
                        beg_inv_units effective_price effective_discount_pct perm_price reg_price currency_id reg_cost con_cost_amt uncon_cost_amt;



         /* get currency_id */
         set work.currency_dm_work key=geo_id;
            if _iorc_ ne 0 then
            do;
              /*
                  This condition should never occur.
                  If it did occur, it is better to move on rather than stop.
               */
            end;

            _iorc_=0;  _error_=0;


      run;
   %end;




   /*  Sort just so the format looks good */
   proc sort data=&_forecast_export_table nodupkey;
      by start_dt end_dt prod_id geo_id;
   run;

   libname _fcstexp  clear;

%mend;

*options mprint mlogic symbolgen;
%*tkmi_job_ro(run_only=fcst_export,etlsaspath=/local/sas/SASFoundation/9.3/misc/dibase/etlsas,debug=1);
