/************************************************************************************************
MERCHANDISE INTELLIGENCE

NAME:           etl_reclass_rpp_plan_validation

DESCRIPTION:    Macro to create the table which will have the information of all the RPP Plans which are invalid due to the following reasons -
1. All children of a node in the hierarchy are reclassed to another parent which results in a dangling node.
The will cause OPT/EVAL to error and prevent the user from correcting the error within the Promotion Optimization Client
2. Products are duplicated in a plan because of a reclass. Plan results are incorrect due to the duplication.
In this scenario the client can be used to correct the issue.
3. Plans become invalid because a product has been reclassified outside of the candidate and/or objective scope.


APPLICATIONS:   RPP

MODULE:

INPUTS:         RPP_PLAN, PROD_SCOPE_DETAIL, PRODUCT_DM, PRODUCT_HIER_ASSOC_DM, RPP_PLAN_PROD_ASGMNT

OUTPUTS:        DI_MON.rpp_reclass_validation_checks

AUTHOR:         sinsks,  Sourav Das

SUPPORT:        sinsks,  Sourav Das

HISTORY:        27Apr16  sinsks   Started creation of macro
*************************************************************************************************/
%macro etl_reclass_rpp_plan_validation;
   /* Setup to capture return codes  */
   %global job_rc trans_rc sqlrc;
   %let sysrc = 0;
   %let job_rc = 0;
   %let trans_rc = 0;
   %let sqlrc = 0;
   %let syscc=0;
   %global syserr;

%macro rcSet(error);
   %if (&error gt &trans_rc) %then
      %let trans_rc = &error;

   %if (&error gt &job_rc) %then
      %let job_rc = &error;
%mend rcSet;

%macro rcSetDS(error);
   if &error gt input(symget('trans_rc'),12.) then
      call symput('trans_rc',trim(left(put(&error,12.))));

   if &error gt input(symget('job_rc'),12.) then
      call symput('job_rc',trim(left(put(&error,12.))));
%mend rcSetDS;

%put ______ Starting the macro etl_reclass_rpp_plan_validation....;
%let load_startTime = %sysfunc(datetime(),datetime.);
%let etls_jobName = etl_reclass_rpp_plan_validation;
%let reclass_dt = today();

/******************************************************************************/
/******        DUPLICATE PRODUCTS IN PRODUCT GROUP                      *******/
/******************************************************************************/
proc sql;
   create table WORK.tmp_dup_table1 as
      select plan_sk,
         prod_hier_sk as parent_prod_hier_sk
      from DI_TRANS.rpp_plan_prod_asgmnt b
         where plan_sk in
            (select distinct plan_sk from DI_TRANS.rpp_plan_prod_asgmnt where product_group_sk > 0)
   ;
quit;

proc sql;
   create table WORK.tmp_dup_table2 as
      select
         d.plan_sk,
         p.parent_prod_hier_sk,
         p.prod_hier_sk,
         p.prod_lvl
      from tmp_dup_table1 d, DI_TRANS.product_hier_assoc_dm p
         where p.parent_prod_hier_sk = d.parent_prod_hier_sk
            and p.prod_hier_assoc_cd = 1;
quit;

proc sql;
   create table WORK.invalid_prod_dups
      as select
         plan_sk,
         prod_hier_sk,
         "Dups" as Type format $10.
      from tmp_dup_table2
         group by prod_hier_sk,plan_sk,prod_lvl
            having count(prod_hier_sk) > 1 and prod_lvl = max(prod_lvl);
quit;

/******************************************************************************/
/******        DANGLING NODES                                           *******/
/******************************************************************************/
proc sql;
   create table WORK.invalid_dangling_nodes
      as select plan_sk, prod_hier_sk,"Dang" as Type format $10.
         from DI_TRANS.rpp_plan_prod_asgmnt
            where prod_hier_sk not in
               (select distinct prod_hier_sk from DI_TRANS.product_hier_assoc_dm where prod_hier_assoc_cd = 1)
                  order by plan_sk;
quit;

/******************************************************************************/
/******        ALL DANGLING NODES                                       *******/
/******************************************************************************/
%etl_update_all_dangling_nodes;

/******************************************************************************/
/******        OUT OF SCOPE PRODUCTS                                    *******/
/******************************************************************************/
%etl_out_of_scope_nodes;

/*** Merge all the tables ***/
data WORK.rpp_reclass_validation_checks;
format type $10.;
   set WORK.invalid_prod_dups WORK.invalid_dangling_nodes final_all_dangling_nodes invalid_out_scope_nods4 invalid_out_scope_nods3 invalid_out_scope_nods2;
run;

proc sort data = WORK.rpp_reclass_validation_checks
   out  = WORK.rpp_reclass_validation_checks
   noduprec;
   by plan_sk;
run;

/*** Merge all the Product SKs ***/
%merge_products

/*** Merge all the tables. ***/
data WORK.rpp_reclass_validation_checks;
   set tmp_invalid_asgmnt_obj_os tmp_invalid_cand_obj_os tmp_invalid_part_os tmp_invalid_prod_dups tmp_all_invalid_dangling_nodes tmp_invalid_dangling_nodes;
run;

proc sort data=rpp_reclass_validation_checks noduprec;
   by plan_sk;

   /*** Fetch the plan name and status code of the affected plans ***/
proc sql;
   create table WORK.rpp_reclass_validation_checks as
      select a.*,
         b.plan_nm,
         b.plan_desc,
         b.status_cd,
         b.notes
      from
         WORK.rpp_reclass_validation_checks a,
         DI_TRANS.rpp_Plan b
      where
         a.plan_sk = b.plan_sk
      order by plan_sk;
quit;

/*** Creating the final table ***/
data WORK.rpp_reclass_validation_checks (keep=plan_sk RC_NOTES NOTES new_notes Plan_nm Plan_Desc Status_Cd Status reclass_dt);
   set WORK.rpp_reclass_validation_checks;
   format RECLASS_DT date9.;
   format STATUS $20.;
   format Notes $255.;
   format RC_NOTES $1000.;
   retain RC_NOTES;

   if first.plan_sk then
      RC_NOTES = "[[" || note;
   else RC_NOTES = catx(" and ", RC_NOTES, note);
   by plan_sk;

   if last.plan_sk;

   if index(Notes,"[[") > 0 then
      do;
         substr(Notes,index(Notes,"[[")) = "";
      end;

   if length(notes) = 0 then
      do;
         Notes = RC_Notes || "]]";
      end;
   else
      do;
         Notes = catx(" ", Notes, RC_NOTES, "]]");
      end;

   RECLASS_DT = today();

   if index(Plan_desc,"_RC_") = 0 then
      do;

         Plan_Desc = catx(" ", "_RC_", Plan_Desc);
      end;

   select ( status_cd );
      when ( 0 )
         do;
            Status = "Draft";
         end;

      when ( 2 )
         do;
            Status = "Approved";
         end;

      when ( 3 )
         do;
            Status = "Executing";
         end;

      when ( 4 )
         do;
            Status = "Executed";
         end;

      otherwise
         do;
            Status = "Tentatively Approved";
         end;
   end;
run;

proc sql noprint;
   select count(plan_sk) into :error_count from WORK.rpp_reclass_validation_checks;
quit;

%di_etl_keep_work_table(rpp_reclass_validation_checks);

/**** Append the table in DI_MON.INVALID_PLANS  ***/
data DI_MON.rpp_reclass_validation_checks;
   set WORK.rpp_reclass_validation_checks;
run;

/***
Update EVAL_OPT_PROCESS_CD to 0 so that UI shows "Out of Date" for the affected plans.
Update EVAL_OPT_REQ_CD to 0 to prevent the optimization of the affected plans through batch process.
***/

/*** Update RPP_PLAN table ***/
%di_dm_trans_update(
   upload_lib=di_mon,
   upload_table=rpp_reclass_validation_checks,
   update_table=rpp_plan,
   sql= %str(update di_trans.rpp_plan p1 set EVAL_OPT_REQ_CD = (select 0 from di_mon.rpp_reclass_validation_checks where p1.plan_sk = plan_sk),
   EVAL_OPT_PROCESS_CD = (select 0 from di_mon.rpp_reclass_validation_checks where p1.plan_sk = plan_sk),
   NOTES = (select Notes from di_mon.rpp_reclass_validation_checks where p1.plan_sk = plan_sk),
   Plan_desc = (select Plan_Desc from di_mon.rpp_reclass_validation_checks where p1.plan_sk = plan_sk)
   where plan_sk in (SELECT plan_sk FROM di_mon.rpp_reclass_validation_checks))
   );
%di_etl_rcset(error=&syscc,code=job_rc);

proc sql noprint;
   drop table WORK.rpp_reclass_validation_checks;
quit;

%if &job_rc = 0 and &error_count = 0 %then
   %let jobstat_msg=%str(Job Successful);
%else %if &job_rc le 4 or &error_count gt 0 %then
   %let jobstat_msg=%str(Job Ended with Data Exceptions or Warnings);
%else %if &job_rc gt 4 %then
   %let jobstat_msg=%str(Job Ended with Errors);
%let etls_endTime = %sysfunc(datetime(),datetime.);
%put updating the job status table;

%di_util_lock(DI_MON.JOB_STATUS);

proc sql noprint;
   insert into DI_MON.JOB_STATUS
      (USER,
      TABLE,
      LIB,
      JOBNAME,
      JOBSTAT,
      JOB_RC,
      RBEFORE,
      RAFTER,
      RSOURCE,
      RINEXCP,
      STTIME,
      ENDTIME)
   values(
      "&sysuserid" ,
      "RECLASS_VALIDATION_RPP_CHECKS" ,
      "DI_MON"  ,
      "&etls_jobName" ,
      "&jobstat_msg" ,
      &job_rc.,
      0,
      &error_count.,
      0,
      &error_count. ,
      "&load_startTime"dt ,
      "&etls_endTime"dt );
quit;

%di_util_unlock(DI_MON.JOB_STATUS);
%put NOTE: macro etl_reclass_rpp_plan_validation started at &load_startTime., ended at &etls_endTime.;
%put ______ Exiting from the macro etl_reclass_rpp_plan_validation....;
%mend;

/*** End of the definition of the main job ***/
/******************************************************************************/
/******        MACRO DEFINITION FOR ALL DANGLING NODES                  *******/
/******************************************************************************/
%macro etl_update_all_dangling_nodes;

   proc sql;
      create table WORK.all_invalid_dangling_nodes
         as select PLAN_SK, prod_hier_sk,count(prod_hier_sk) as dang_prods
            from DI_TRANS.rpp_plan_prod_asgmnt
               where prod_hier_sk not in
                  (select distinct prod_hier_sk from DI_TRANS.product_hier_assoc_dm where prod_hier_assoc_cd = 1)
                     group by plan_sk;
   quit;

   proc sql;
      create table WORK.all_plan_products
         as select plan_sk, prod_hier_sk, count(prod_hier_sk) as all_prods
            from DI_TRANS.rpp_plan_prod_asgmnt
               group by plan_sk;
   quit;

   proc sql;
      create table work.tmp_all_dangling_1 as
         select distinct a.plan_sk,a.prod_hier_sk
            from WORK.all_plan_products a, WORK.all_invalid_dangling_nodes b
               where a.plan_Sk = b.plan_sk and all_prods = dang_prods
                  group by
                     a.plan_sk;
   quit;

   proc sql;
      create table work.hierarchy_history as select hier_sk,prev_parent_sk,new_parent_sk from
         di_dm.hierarchy_history
      where hier_type = 1
         group by prev_parent_sk
            having hier_change_dt = max(hier_change_dt);
   quit;

   proc sql;
      create table work.tmp_all_dangling_2 as
         select distinct a.*,b.new_parent_sk
            from
               work.tmp_all_dangling_1 a,
               work.hierarchy_history b
            where a.prod_hier_sk = b.prev_parent_sk
               order by plan_sk;
   quit;

   proc sql noprint;
      create table tmp_plans as select distinct plan_sk from work.tmp_all_dangling_2;
   quit;

   data tmp_plans(keep=plan_sk obs);
      set tmp_plans;
      obs = _N_;
      by plan_sk;

      if first.plan_sk;
   run;

   proc sql noprint;
      select count(distinct plan_sk) into :cnt from work.tmp_all_dangling_2;
   quit;

   data all_dangling_nodes;
      format plan_sk 8.;
      format prod_hier_sk 8.;
      format parent_prod_hier_sk 8.;
      stop;
   run;

   %do i=1 %to &cnt.;

      proc sql noprint;
         select distinct plan_sk into :plan_sk from work.tmp_plans where obs = &i;
      quit;

      %let plan_sk = &plan_sk;
      %put Running for Plan &plan_sk;

      proc sql noprint;
         select distinct new_parent_sk into :prod_list separated by ',' from work.tmp_all_dangling_2 where plan_sk = &plan_sk;
      quit;

      proc sql;
         create table tmp_all_dangling_3 as
            SELECT distinct prod_hier_sk, parent_prod_hier_sk,
               Count(* ) as parent_sk_count,
               Min(parent_prod_lvl) as min_lvl
            FROM  di_trans.product_hier_assoc_dm phad
               WHERE prod_hier_sk IN (&prod_list.) and prod_hier_sk ne parent_prod_hier_sk
                  AND prod_hier_assoc_cd = 1
               GROUP BY parent_prod_hier_sk;
      quit;

      proc sql;
         create table tmp_all_dangling_4 as select * from work.tmp_all_dangling_3 group by prod_hier_sk having parent_sk_count = max(parent_sk_count);
      quit;

      proc sql;
         create table tmp_&plan_sk as select &plan_sk as plan_sk, prod_hier_sk,parent_prod_hier_sk from work.tmp_all_dangling_4 having min_lvl = max(min_lvl);
      quit;

      proc append data = tmp_&plan_sk base = all_dangling_nodes;
      quit;

   %end;

   proc sql;
      create table work.plan_geo_scope as
         select distinct dng.plan_sk,
            parent_prod_hier_sk as prod_hier_sk,
            coalesce(a.objective_sk, a.scope_sk) as scope_sk
         from di_trans.rpp_plan as a,
            work.all_dangling_nodes as dng
         where a.plan_sk eq dng.plan_sk;
   quit;

   proc sql;
      create table work.distinct_scope_tocrt as
         select distinct prod_hier_sk,
            scope_sk
         from work.plan_geo_scope;
   quit;

   proc sql;
      create table work.distinct_scope_geo_tocrt as
         select a.prod_hier_sk,
            a.scope_sk,
            b.geo_hier_sk
         from work.distinct_scope_tocrt a ,
            di_trans.geo_scope_detail b
         where a.scope_sk eq b.scope_sk
            order by prod_hier_sk , scope_sk
      ;
   quit;

   data work.scope_final(keep=SCOPE_SK SCOPE_NM SCOPE_DESC PROD_HIER_ASSOC_CD GEO_HIER_ASSOC_CD)
      work.scope_detail(keep=SCOPE_NM prod_hier_sk geo_hier_sk);
      set work.distinct_scope_geo_tocrt;
      by prod_hier_sk scope_sk;
      SCOPE_NM = catx("_","ETL_GEN",scope_sk,prod_hier_sk);

      if last.scope_sk then
         do;
            SCOPE_DESC = SCOPE_NM;
            PROD_HIER_ASSOC_CD = 1;
            GEO_HIER_ASSOC_CD = 1;
            output work.scope_final;
         end;

      output  work.scope_detail;
   run;

   proc sql;
      create table work.scope_final_not_exist as
         select  SCOPE_NM, SCOPE_DESC, PROD_HIER_ASSOC_CD, GEO_HIER_ASSOC_CD
            from work.scope_final where SCOPE_NM not in ( select SCOPE_NM from di_trans.scope);
   quit
      quit;

   %di_dm_trans_append(
      di_trans=di_dm2,
      trans_table=scope,
      append_table=work.scope_final_not_exist,
      bulkload=0,
      fk_flag=0,
      upload_flag=0
      );
   %di_dm_trans_extract(
      di_trans      = di_dm2,
      trans_table   = scope,
      all_trans     = 0,
      select_list   = %str(A.scope_nm,max(A.scope_sk) as scope_sk),
      filter_table  = work.scope_final_not_exist,
      filter_cond   = %str(A.scope_nm = B.scope_nm group by A.scope_nm order by A.scope_nm),
      _output_table = recent_scope_sk
      );

   proc sql;
      create table work.scope_detail_append as
         select b.scope_sk,
            a.prod_hier_sk,
            a.geo_hier_sk
         from
            work.scope_detail a,
            work.recent_scope_sk b
         where a.scope_nm eq b.scope_nm;
   quit;

   %di_etl_rcSet(error=&syscc,code=job_rc);
   %di_dm_trans_append(
      di_trans=di_dm2,
      trans_table=Scope_detail,
      append_table=scope_detail_append,
      bulkload=0,
      fk_flag=0,
      upload_flag=0
      );

   data work.s_dangling_int(keep=scope_nm)
      work.gg_dangling_int
   ;
      set work.plan_geo_scope;
      SCOPE_NM = catx("_","ETL_GEN",scope_sk,prod_hier_sk);
   run;

   %di_dm_trans_extract(
      di_trans      = di_dm2,
      trans_table   = scope,
      all_trans     = 0,
      select_list   = %str(A.scope_nm,max(A.scope_sk) as scope_sk),
      filter_table  = work.s_dangling_int,
      filter_cond   = %str(A.scope_nm = B.scope_nm group by A.scope_nm order by A.scope_nm),
      _output_table = last_scope_sk
      );

   proc sql;
      create table work.final_all_dangling_nodes
         as select distinct a.plan_sk,
            new.scope_sk,
            a.scope_sk as prev_scope_sk,
            "all_dang" as Type
         from work.gg_dangling_int as a,
            work.last_scope_sk new
         where a.SCOPE_NM eq new.SCOPE_NM;
   quit;

   /*** Update RPP_PLAN table only plans that are not executing (3) or executed (4). It will update "Draft", "Approved" and "Tentatively Approved" ***/
   %di_dm_trans_update(
      upload_lib=work,
      upload_table=final_all_dangling_nodes,
      update_table=rpp_plan,
      sql= %str(update di_trans.rpp_plan p1 set EVAL_OPT_REQ_CD = (select 0 from work.final_all_dangling_nodes where p1.plan_sk = plan_sk),
      EVAL_OPT_PROCESS_CD = (select 0 from work.final_all_dangling_nodes where p1.plan_sk = plan_sk),
      SCOPE_SK = (select SCOPE_SK from work.final_all_dangling_nodes where p1.plan_sk = plan_sk),
      OBJECTIVE_SK = (select SCOPE_SK from work.final_all_dangling_nodes where p1.plan_sk = plan_sk)
      where plan_sk in (SELECT plan_sk FROM work.final_all_dangling_nodes where status_cd < 3))
      );

   /*** Update PROD_SCOPE_DETAIL ***/
   %di_dm_trans_delete(di_trans=di_trans,trans_table=PROD_SCOPE_DETAIL);
   %di_dm_trans_exe(
      pass_through_sql=%str(insert into di_trans.PROD_SCOPE_DETAIL (scope_sk, prod_hier_sk)
      select distinct scope_sk, prod_hier_sk from di_trans.scope_detail)
      );

   /*** Update GEO_SCOPE_DETAIL ***/
   %di_dm_trans_delete(di_trans=di_trans,trans_table=GEO_SCOPE_DETAIL);
   %di_dm_trans_exe(
      pass_through_sql=%str(insert into di_trans.GEO_SCOPE_DETAIL (scope_sk, geo_hier_sk)
      select distinct scope_sk, geo_hier_sk from di_trans.scope_detail)
      );
%mend;

/******************************************************************************/
/******        MACRO DEFINITION FOR OUT OF SCOPE PRODUCTS               *******/
/******************************************************************************/
%macro etl_out_of_scope_nodes;
   %local prod_partition_lvl;

   proc sql noprint;
      select spec_value into :prod_partition_lvl from di_data.global_setting
         where upcase(spec_nm) = 'PROD_PARTITION_LVL';
   quit;

   /*   %let sql_st = %bquote(select plan.plan_sk,det.prod_hier_sk from DI_TRANS.Rpp_plan plan, DI_TRANS.Prod_scope_detail det,*/
   /*      DI_TRANS.product_hier_assoc_dm p Where plan.objective_sk = det.scope_sk and det.prod_hier_sk = p.prod_hier_sk*/
   /*      and p.prod_hier_assoc_cd = 1 and p.parent_prod_lvl = &prod_partition_lvl*/
   /*      group by plan.plan_sk,det.prod_hier_sk having count(distinct(parent_prod_hier_sk)) > 1);*/
   /**/
   /*   %di_dm_trans_exe(*/
   /*      pass_through_sql=%str(&sql_st),*/
   /*      _output_table=invalid_out_scope_nods1,*/
   /*      debug_flag=1);*/
   %local except_opr;

   %if %upcase(&di_trans_vendor) eq SAS %then
      %let except_opr = except;
   %else %let except_opr = minus;
   %let sql_st = %bquote(select p1.plan_sk,Scope_Det.prod_hier_sk from DI_TRANS.Rpp_plan p1,DI_TRANS.Prod_scope_detail scope_Det
      Where p1.scope_sk = scope_det.scope_sk and p1.scope_sk <> p1.objective_sk and p1.objective_sk is Not Null )
      &except_opr
      %bquote(select p1.plan_sk,Scope_Det.prod_hier_sk from DI_TRANS.Rpp_plan p1,DI_TRANS.Prod_scope_detail scope_Det,
      DI_TRANS.Prod_scope_detail obj_det, DI_TRANS.product_hier_assoc_dm ph Where p1.scope_sk <> p1.objective_sk
      and p1.objective_sk is Not Null and p1.scope_sk = scope_Det.scope_sk and p1.objective_sk = obj_det.scope_sk
      and obj_det.prod_hier_sk = ph.parent_prod_hier_sk and Scope_Det.prod_hier_sk = ph.prod_hier_sk
      and ph.prod_hier_assoc_cd = 1);

   %di_dm_trans_exe(
      pass_through_sql=%str(&sql_st),
      _output_table=invalid_out_scope_nods2,
      debug_flag=1);

   /* Find out plan-prod-assignments which dont fall under the children of objective scope */
   %let sql_st = %bquote(select pln.plan_sk, rpp.prod_hier_sk from DI_TRANS.rpp_plan_prod_asgmnt rpp , DI_TRANS.rpp_plan pln
      where pln.plan_sk = rpp.plan_sk
      &except_opr
      select pp.plan_sk, pp.prod_hier_sk from DI_TRANS.Rpp_plan_prod_asgmnt pp,
      DI_TRANS.product_hier_assoc_dm ph,DI_TRANS.Rpp_plan plan, DI_TRANS.Prod_scope_detail Det
      Where plan.plan_sk = pp.plan_sk and plan.objective_sk = Det.scope_sk
      and Det.prod_hier_sk = ph.parent_prod_hier_sk
      and pp.prod_hier_sk = ph.prod_hier_sk and ph.prod_hier_assoc_cd = 1);

   %di_dm_trans_exe(
      pass_through_sql=%str(&sql_st),
      _output_table=invalid_out_scope_nods3,
      debug_flag=1);
   %let sql_st = %bquote(select distinct pp.plan_sk, ph.parent_prod_hier_sk as parent_sk
      from DI_TRANS.Rpp_plan_prod_asgmnt pp,DI_TRANS.rpp_plan pln, DI_TRANS.product_hier_assoc_dm ph
      Where pp.plan_sk = pln.plan_sk and pp.prod_hier_sk = ph.prod_hier_sk
      and ph.prod_hier_assoc_cd = 1 and ph.parent_prod_lvl = &prod_partition_lvl);

   %di_dm_trans_exe(
      pass_through_sql=%str(&sql_st),
      _output_table=invalid_out_scope_nods41,
      debug_flag=1);

   proc sql;
      create table invalid_out_scope_nods4
         as
            select plan_sk,
               parent_sk as prod_hier_sk,
               "out_part" as Type
            from
               invalid_out_scope_nods41
            where
               plan_sk
               in
            ( select plan_sk from invalid_out_scope_nods41 group by plan_sk having count(*) > 1 )
               order by plan_sk;
   quit;

   data invalid_out_scope_nods3;
      set invalid_out_scope_nods3;
      type = "asgmnt_os";
   run;

   data invalid_out_scope_nods2;
      set invalid_out_scope_nods2;
      type = "cand_obj";
   run;

%mend;

%macro merge_products;
   /*** Fetch the product IDs and Product Names ***/
   proc sql;
      create table WORK.rpp_reclass_validation_checks
         as select
            a.plan_sk,
            a.prod_hier_sk,
            b.prod_id,
            b.prod_nm,
            a.Type,
            a.scope_sk,
            a.prev_scope_sk
         from rpp_reclass_validation_checks a
            left join DI_TRANS.product_dm b
               on a.prod_hier_sk = b.prod_hier_sk
            order by plan_sk;
   quit;

   /*** Merge the product IDs ***/
   data tmp_invalid_dangling_nodes (keep=plan_sk Note );
      set WORK.rpp_reclass_validation_checks;
      format prod_ids $900.;
      retain prod_ids;
      where Type = "Dang";

      if first.plan_sk then
         prod_ids = prod_id;
      else prod_ids = cats(prod_ids,',', prod_id);
      by plan_sk;
      Note = " Dangling Product(s) - " || prod_ids;

      if last.plan_sk;
   run;

   data tmp_all_invalid_dangling_nodes (keep=plan_sk Note scope_sk prev_scope_sk );
      set WORK.rpp_reclass_validation_checks;
      format prod_ids $900.;
      retain prod_ids;
      where Type = "all_dang";
      by plan_sk;
      Note = catx(" ","Scope SK changed from",prev_scope_sk,"to",scope_sk);

      if last.plan_sk;
   run;

   data tmp_invalid_prod_dups (keep=plan_sk Note );
      set WORK.rpp_reclass_validation_checks;
      format prod_ids $900.;
      retain prod_ids;
      where Type = "Dups";

      if first.plan_sk then
         prod_ids = prod_id;
      else prod_ids = cats(prod_ids,',', prod_id);
      by plan_sk;
      Note = " Duplicate Product(s)- " || prod_ids;

      if last.plan_sk;
   run;

   data tmp_invalid_part_os (keep=plan_sk Note );
      set WORK.rpp_reclass_validation_checks;
      format prod_ids $900.;
      retain prod_ids;
      where Type = "out_part";

      if first.plan_sk then
         prod_ids = prod_id;
      else prod_ids = cats(prod_ids,',', prod_id);
      by plan_sk;
      Note = " Products across partitions - " || prod_ids;

      if last.plan_sk;
   run;

   data tmp_invalid_cand_obj_os (keep=plan_sk Note );
      set WORK.rpp_reclass_validation_checks;
      format prod_ids $900.;
      retain prod_ids;
      where Type = "cand_obj";

      if first.plan_sk then
         prod_ids = prod_id;
      else prod_ids = cats(prod_ids,',', prod_id);
      by plan_sk;
      Note = " Candidate scope not within objective scope - " || prod_ids;

      if last.plan_sk;
   run;

   data tmp_invalid_asgmnt_obj_os (keep=plan_sk Note );
      set WORK.rpp_reclass_validation_checks;
      format prod_ids $900.;
      retain prod_ids;
      where Type = "asgmnt_os";

      if first.plan_sk then
         prod_ids = prod_id;
      else prod_ids = cats(prod_ids,',', prod_id);
      by plan_sk;
      Note = " Plan asgmnts not within objective scope - " || prod_ids;

      if last.plan_sk;
   run;

%mend;

/*** End of Merge_products macro ***/
